@@ -28,10 +28,6 @@ use strict;
 use warnings;
 use autodie;
 use v5.10;
-use Carp qw(croak confess);
-
-use Cwd();
-use File::Find;
 
 # Half of the size used in the "sliding window" for detecting bad
 # licenses like GFDL with invariant sections.
@@ -41,7 +37,7 @@ use constant BLOCKSIZE => 4096;
 use Lintian::Data;
 use Lintian::Relation ();
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(fail is_ancestor_of normalize_pkg_path strip);
+use Lintian::Util qw(fail normalize_pkg_path strip);
 use Lintian::SlidingWindow;
 
 # All the packages that may provide config.{sub,guess} during the build, used
@@ -105,7 +101,7 @@ my $WARN_FILE_TYPE =  Lintian::Data->new(
         $transformlist = strip($transformlist);
 
         my $syntaxerror = 'Syntax error in cruft/warn-file-type';
-        my @transformpairs = ();
+        my @transformpairs;
         unless($transformlist eq '') {
             my @transforms = split(/\s*\&\&\s*/, $transformlist);
             if(scalar(@transforms) > 0) {
@@ -180,6 +176,7 @@ sub _get_license_check_file {
                   \&_check_gfdl_license_problem,
                 'rfc-whitelist-filename' =>\&_rfc_whitelist_filename,
                 'php-source-whitelist' => \&_php_source_whitelist,
+                #'print-group'          => sub { print($1)},
             );
             my @splitline = split(/\s*\~\~\s*/, $_[1], 5);
             my $syntaxerror = 'Syntax error in '.$filename;
@@ -280,19 +277,19 @@ our %ERRORS = (
 # source-contains or diff-contains).  Note that only one of these regexes
 # should trigger for any single directory.
 my @directory_checks = (
-    [qr,^(.+/)?CVS$,        => 'cvs-control-dir'],
-    [qr,^(.+/)?\.svn$,      => 'svn-control-dir'],
-    [qr,^(.+/)?\.bzr$,      => 'bzr-control-dir'],
-    [qr,^(.+/)?\{arch\}$,   => 'arch-control-dir'],
-    [qr,^(.+/)?\.arch-ids$, => 'arch-control-dir'],
-    [qr!^(.+/)?,,.+$!       => 'arch-control-dir'],
-    [qr,^(.+/)?\.git$,      => 'git-control-dir'],
-    [qr,^(.+/)?\.hg$,       => 'hg-control-dir'],
-    [qr,^(.+/)?\.be$,       => 'bts-control-dir'],
-    [qr,^(.+/)?\.ditrack$,  => 'bts-control-dir'],
+    [qr,^(.+/)?CVS/?$,        => 'cvs-control-dir'],
+    [qr,^(.+/)?\.svn/?$,      => 'svn-control-dir'],
+    [qr,^(.+/)?\.bzr/?$,      => 'bzr-control-dir'],
+    [qr,^(.+/)?\{arch\}/?$,   => 'arch-control-dir'],
+    [qr,^(.+/)?\.arch-ids/?$, => 'arch-control-dir'],
+    [qr!^(.+/)?,,.+/?$!       => 'arch-control-dir'],
+    [qr,^(.+/)?\.git/?$,      => 'git-control-dir'],
+    [qr,^(.+/)?\.hg/?$,       => 'hg-control-dir'],
+    [qr,^(.+/)?\.be/?$,       => 'bts-control-dir'],
+    [qr,^(.+/)?\.ditrack/?$,  => 'bts-control-dir'],
 
     # Special case (can only be triggered for diffs)
-    [qr,^(.+/)?\.pc$, => 'quilt-control-dir'],
+    [qr,^(.+/)?\.pc/?$, => 'quilt-control-dir'],
 );
 
 # File checks.  These regexes match files that shouldn't be in the source
@@ -319,19 +316,16 @@ our @EOL_TERMINATORS_FILES = qw(control changelog);
 sub run {
     my (undef, undef, $info, $proc) = @_;
     my $source_pkg = $proc->pkg_src;
-    my $droot = $info->debfiles;
+    my $d_files = $info->index_resolved_path('debian/files');
 
-    if (-e "$droot/files" and not -z "$droot/files") {
+    if ($d_files and $d_files->is_file and $d_files->size != 0) {
         tag 'debian-files-list-in-source';
     }
 
     # This doens't really belong here, but there isn't a better place at the
     # moment to put this check.
-    my $version = $info->field('version');
+    my $version = $info->field('version', '0-1');
 
-    # If the version field is missing, assume it to be a native,
-    # maintainer upload as it is probably the most likely case.
-    $version = '0-1' unless defined $version;
     if ($info->native) {
         if ($version =~ /-/ and $version !~ /-0\.[^-]+$/) {
             tag 'native-package-with-dash-version';
@@ -346,11 +340,6 @@ sub run {
     # or libtool.
     my $atdinbd= $info->relation('build-depends-all')->implies($AUTOTOOLS);
     my $ltinbd  = $info->relation('build-depends-all')->implies($LIBTOOL);
-
-    # Create a closure so that we can pass our lexical variables into
-    # the find wanted function.  We don't want to make them global
-    # because we'll then leak that data across packages in a large
-    # Lintian run.
     my %warned;
     my $format = $info->field('format');
 
@@ -360,20 +349,19 @@ sub run {
     #   that may not be present.
     $format = '3.0 (quilt)' unless defined $format;
     if ($format =~ /^\s*2\.0\s*\z/ or $format =~ /^\s*3\.0\s*\(quilt\)/) {
-        my $wanted= sub { check_debfiles($info, qr/\Q$droot\E/, \%warned) };
-        find($wanted, $droot);
+        check_debian_dir($info, \%warned);
     }elsif (not $info->native) {
         check_diffstat($info->diffstat, \%warned);
     }
     find_cruft($source_pkg, $info, \%warned, $atdinbd, $ltinbd);
 
     for my $file (@EOL_TERMINATORS_FILES) {
-        my $path = $info->debfiles($file);
-        next if not -f $path or not is_ancestor_of($droot, $path);
-        open(my $fd, '<', $path);
+        my $path = $info->index_resolved_path("debian/$file");
+        next if not $path or not $path->is_open_ok;
+        my $fd = $path->open;
         while (my $line = <$fd>) {
             if ($line =~ m{ \r \n \Z}xsm) {
-                tag 'control-file-with-CRLF-EOLs', "debian/$file";
+                tag 'control-file-with-CRLF-EOLs', $path;
                 last;
             }
         }
@@ -480,34 +468,50 @@ sub check_diffstat {
 # 3.0 (quilt) packages where there is no Debian diff and hence no diffstat
 # output.  Record any files we warn about in $warned so that we don't warn
 # again when checking the full unpacked source.
-sub check_debfiles {
-    my ($info, $droot, $warned) = @_;
-    (my $name = $File::Find::name) =~ s,^$droot/,,;
+sub check_debian_dir {
+    my ($info, $warned) = @_;
+    my $droot = $info->index_resolved_path('debian/');
+    return if not $droot;
 
-    # Check for unwanted directories and files.  This really duplicates the
-    # find_cruft function and we should find a way to combine them.
-    if (-d) {
-        for my $rule (@directory_checks) {
-            if ($name =~ /$rule->[0]/) {
-                tag "diff-contains-$rule->[1]", "debian/$name";
-                $warned->{"debian/$name"} = 1;
+    my @worklist = $droot->children;
+
+    # Check for unwanted directories and files.  This really
+    # duplicates the find_cruft function and we should find a way to
+    # combine them.
+
+  ENTRY:
+    while (my $entry = shift(@worklist)) {
+        my $name     = $entry->name;
+
+        if ($entry->is_dir) {
+            # Remove the trailing slash (historically we never
+            # included the slash for these tags and there is no
+            # particular reason to change that now).
+            $name     = substr($name,     0, -1);
+
+            for my $rule (@directory_checks) {
+                if ($name =~ /$rule->[0]/) {
+                    tag "diff-contains-$rule->[1]", $name;
+                    $warned->{$name} = 1;
+                }
+            }
+            push(@worklist, $entry->children);
+        } else {
+
+            for my $rule (@file_checks) {
+                if ($name =~ /$rule->[0]/) {
+                    tag "diff-contains-$rule->[1]", $entry;
+                    $warned->{$name} = 1;
+                }
+            }
+
+            # Additional special checks only for the diff, not the
+            # full source.
+            if ($name =~ m@^debian/(?:.+\.)?substvars$@o) {
+                tag 'diff-contains-substvars', $entry;
             }
         }
     }
-
-    -f or return;
-
-    for my $rule (@file_checks) {
-        if ($name =~ /$rule->[0]/) {
-            tag "diff-contains-$rule->[1]", "debian/$name";
-            $warned->{"debian/$name"} = 1;
-        }
-    }
-
-    # Additional special checks only for the diff, not the full source.
-    if ($name =~ m@^(?:.+\.)?substvars$@o) {
-        tag 'diff-contains-substvars', "debian/$name";
-    }
     return;
 }
 
@@ -531,7 +535,6 @@ sub find_cruft {
         my $name     = $entry->name;
         my $basename = $entry->basename;
         my $dirname = $entry->dirname;
-        my $path;
         my $file_info;
 
         if ($entry->is_dir) {
@@ -618,9 +621,8 @@ sub find_cruft {
 
         # waf is not allowed
         if ($basename =~ /\bwaf$/) {
-            my $path   = $info->unpacked($entry);
             my $marker = 0;
-            open(my $fd, '<', $path);
+            my $fd = $entry->open;
             while (my $line = <$fd>) {
                 next unless $line =~ m/^#/o;
                 if ($marker && $line =~ m/^#BZ[h0][0-9]/o) {
@@ -642,6 +644,13 @@ sub find_cruft {
             tag 'debian-upstream-obsolete-path', $name;
         }
 
+        if (   $basename eq 'doxygen.png'
+            or $basename eq 'doxygen.sty') {
+            unless ($source_pkg eq 'doxygen') {
+                tag 'source-contains-prebuilt-doxygen-documentation', $dirname;
+            }
+        }
+
         unless ($warned->{$name}) {
             for my $rule (@file_checks) {
                 next if ($rule->[2] and not $info->native);
@@ -660,7 +669,7 @@ sub find_cruft {
             }
         }elsif ($basename =~ m{\A config.(?:guess|sub) \Z}xsm
             and not $atdinbd){
-            open(my $fd, '<', $info->unpacked($entry));
+            my $fd = $entry->open;
             while (<$fd>) {
                 last
                   if $.> 10;  # it's on the 6th line, but be a bit more lenient
@@ -683,7 +692,7 @@ sub find_cruft {
         }elsif ($basename eq 'ltconfig' and not $ltinbd) {
             tag 'ancient-libtool', $name;
         }elsif ($basename eq 'ltmain.sh', and not $ltinbd) {
-            open(my $fd, '<', $info->unpacked($entry));
+            my $fd = $entry->open;
             while (<$fd>) {
                 if (/^VERSION=[\"\']?(1\.(\d)\.(\d+)(?:-(\d))?)/) {
                     my ($version, $major, $minor, $debian)=($1, $2, $3, $4);
@@ -697,8 +706,7 @@ sub find_cruft {
             }
             close($fd);
         }
-        full_text_check($source_pkg, $entry, $info, $name, $basename, $dirname,
-            $info->unpacked($entry));
+        full_text_check($source_pkg, $entry, $info, $name, $basename,$dirname);
     }
     return;
 }
@@ -774,7 +782,7 @@ sub check_missing_source {
 # note that it does not replace licensecheck(1)
 # and is only used for autoreject by ftp-master
 sub full_text_check {
-    my ($source_pkg, $entry, $info, $name, $basename, $dirname, $path) = @_;
+    my ($source_pkg, $entry, $info, $name, $basename, $dirname) = @_;
 
     # license string in debian/changelog are probably just change
     # Ignore these strings in d/README.{Debian,source}.  If they
@@ -786,7 +794,7 @@ sub full_text_check {
         return;
     }
 
-    open(my $fd, '<:raw', $path);
+    my $fd = $entry->open(':raw');
     # allow to check only text file
     unless (-T $fd) {
         close($fd);
@@ -835,7 +843,7 @@ sub full_text_check {
         # check only in block 0
         if($blocknumber == 0) {
             _search_in_block0($entry, $info, $name, $basename, $dirname,
-                $path, $block);
+                $block);
         }
     }
     close($fd);
@@ -855,7 +863,7 @@ sub _is_javascript_but_not_minified {
 
 # search something in block $0
 sub _search_in_block0 {
-    my ($entry, $info, $name, $basename, $dirname, $path, $block) = @_;
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
 
     if(_is_javascript_but_not_minified($name)) {
         # exception sphinx documentation
@@ -875,15 +883,14 @@ sub _search_in_block0 {
             }
         }
         # now search hidden minified
-        _linelength_test($entry, $info, $name, $basename, $dirname,
-            $path, $block);
+        _linelength_test($entry, $info, $name, $basename, $dirname,$block);
     }
     return;
 }
 
 # try to detect non human source based on line length
 sub _linelength_test {
-    my ($entry, $info, $name, $basename, $dirname, $path, $block) = @_;
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
     my $strip = $block;
     # from perl faq strip comments
     $strip
@@ -903,7 +910,7 @@ sub _linelength_test {
             # Check for missing source.  It will check
             # for the source file in well known directories
             check_missing_source($entry,$info,$name,$basename,$dirname,
-                [['','']]);
+                [['(?i)\.js$','.debug.js'],['(?i)\.js$','-debug.js'],['','']]);
         }
     }
     return;
