@@ -354,7 +354,7 @@ sub run {
         my $march = $info->field('multi-arch');
         unfold('multi-arch', \$march);
         tag 'unknown-multi-arch-value', $pkg, $march
-          unless $march =~ m/^no|foreign|allowed|same$/o;
+          unless $march =~ m/^(?:no|foreign|allowed|same)$/o;
         if (   $march eq 'same'
             && $type eq 'binary'
             && defined $info->field('architecture')) {
@@ -368,7 +368,8 @@ sub run {
     if ($type eq 'binary'){
         if ($pkg =~ /^fonts-/) {
             tag 'font-package-not-multi-arch-foreign'
-              unless $info->field('multi-arch', 'no') =~ m/^foreign|allowed$/o;
+              unless $info->field('multi-arch', 'no')
+              =~ m/^(?:foreign|allowed)$/o;
         }
     }
 
@@ -613,7 +614,7 @@ sub run {
         my $uri = URI->new($homepage);
 
         # not an absolute URI or (most likely) an invalid protocol
-        unless ($uri->scheme && $uri->scheme =~ m/^ftp|https?|gopher$/o) {
+        unless ($uri->scheme && $uri->scheme =~ m/^(?:ftp|https?|gopher)$/o) {
             tag 'bad-homepage', $orig;
         }
 
@@ -674,7 +675,7 @@ sub run {
     if (($type eq 'binary') || ($type eq 'udeb')) {
         my $javalib = 0;
         my $replaces = $info->relation('replaces');
-        my %nag_once = ();
+        my %nag_once;
         $javalib = 1 if($pkg =~ m/^lib.*-java$/o);
         for my $field (
             qw(depends pre-depends recommends suggests conflicts provides enhances replaces breaks)
@@ -1355,7 +1356,7 @@ sub _split_dep {
 
 sub _load_dependency_restrictions {
     my ($key, $value, $pval) = @_;
-    my $ret = undef;
+    my $ret;
     if (not defined $pval) {
         $ret = $pval = [];
     }
