@@ -75,7 +75,7 @@ my $PRIVACY_BREAKER_TAG_ATTR= Lintian::Data->new(
     sub {
         my ($keywords,$regex) = split(/\s*\~\~\s*/, $_[1], 2);
         $regex =~ s/&URL/(?:ht|f)tps?:\/\/[^"\r\n]*/g;
-        my @keywordlist = ();
+        my @keywordlist;
         my @keywordsorraw = split(/\s*\|\|\s*/,$keywords);
         foreach my $keywordor (@keywordsorraw) {
             my @keywordsandraw = split(/\s*&&\s*/,$keywordor);
@@ -198,7 +198,7 @@ sub _detect_embeded_libraries {
                 }
                 if(defined($reinside)) {
                     my $foundre = 0;
-                    open(my $fd, '<:raw', $info->unpacked($file));
+                    my $fd = $file->open(':raw');
 
                     my $sfd
                       = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); });
@@ -470,7 +470,7 @@ sub run {
             #----------------- /etc/php5/conf.d
             elsif ($fname =~ m,^etc/php5/conf.d/.+\.ini$,) {
                 if ($file->is_file) {
-                    open(my $fd, '<', $info->unpacked($file));
+                    my $fd = $file->open;
                     while (<$fd>) {
                         next unless (m/^\s*#/);
                         tag 'obsolete-comments-style-in-php-ini', $file;
@@ -547,7 +547,7 @@ sub run {
                         and $file->size <= 276
                         and $file->is_file
                         and $info->file_info($file) =~ m/gzip compressed/) {
-                        my $fd = open_gz($info->unpacked($file));
+                        my $fd = $file->open_gz;
                         my $f = <$fd>;
                         close($fd);
                         unless (defined $f and length $f) {
@@ -594,7 +594,7 @@ sub run {
                 my $pkg_config_arch = $1 // '';
                 $pkg_config_arch =~ s,\A/,,ms;
 
-                open(my $fd, '<:raw', $info->unpacked($file));
+                my $fd = $file->open(':raw');
                 my $sfd = Lintian::SlidingWindow->new($fd);
               BLOCK:
                 while (my $block = $sfd->readwindow()) {
@@ -1042,7 +1042,7 @@ sub run {
                 && $fname =~ m,\.pm$,
                 && !$dep->implies(
                     'libperl4-corelibs-perl | perl (<< 5.12.3-7)')) {
-                open(my $fd, '<', $info->unpacked($file));
+                my $fd = $file->open;
                 while (<$fd>) {
                     if (
                         m{ (?:do|require)\s+(?:'|") # do/require
@@ -1149,9 +1149,8 @@ sub run {
         }
 
         # doxygen map + md5sum
-        if (   $fname =~ m,^usr/share/doc/$ppkg/[^/]+/.+\.md5$,
-            or $fname =~ m,^usr/share/doc/$ppkg/[^/]+/.+\.map$,) {
-            if($info->index($file->dirname.'doxygen.png')) {
+        if ($fname =~ m,^usr/share/doc/$ppkg/[^/]+/.+\.md5$,) {
+            if ($file->parent_dir->child('doxygen.png')) {
                 tag 'useless-autogenerated-doxygen-file', $file;
             }
         }
@@ -1186,7 +1185,7 @@ sub run {
           ){
             my ($debug, $pyver, $loc, $rest) = ($1, $2, $3, $4);
             my ($pmaj, $pmin) = split(m/\./o, $pyver, 2);
-            my @correction = ();
+            my @correction;
             $pmin = 0 unless (defined $pmin);
             $debug = '' unless (defined $debug);
             next if ($pmaj < 2 or $pmaj > 3); # Not python 2 or 3
@@ -1303,7 +1302,7 @@ sub run {
             # ---------------- embedded Feedparser library
             if (    $fname =~ m,/feedparser\.py$,
                 and $source_pkg ne 'feedparser'){
-                open(my $fd, '<', $info->unpacked($file));
+                my $fd = $file->open;
                 while (<$fd>) {
                     if (m,Universal feed parser,) {
                         tag 'embedded-feedparser-library', $file;
@@ -1328,7 +1327,7 @@ sub run {
                 }
                 my $finfo = $info->file_info($file) || '';
                 if ($finfo =~ m/PostScript Type 1 font program data/) {
-                    my $path = $info->unpacked($file);
+                    my $path = $file->fs_path;
                     my $foundadobeline = 0;
                     open(my $t1pipe, '-|', 't1disasm', $path);
                     while (my $line = <$t1pipe>) {
@@ -1421,10 +1420,8 @@ sub run {
                 if ($finfo !~ m/gzip compressed/) {
                     tag 'gz-file-not-gzip', $file;
                 } else {
-                    my $path = $info->unpacked($file);
-                    my $buff;
-                    my $mtime;
-                    open(my $fd, '<', $path);
+                    my ($buff, $mtime);
+                    my $fd = $file->open;
                     # We need to read at least 8 bytes
                     if (sysread($fd, $buff, 1024) >= 8) {
                         # Extract the flags and the mtime.
@@ -1670,8 +1667,7 @@ sub run {
                 my @linkcomponents = split('/', $mylink);
 
                 # handle `../' at beginning of $link
-                my $lastpop = undef;
-                my $linkcomponent = undef;
+                my ($lastpop, $linkcomponent);
                 while ($linkcomponent = shift @linkcomponents) {
                     if ($linkcomponent eq '.') {
                         tag 'symlink-contains-spurious-segments',
@@ -1916,7 +1912,7 @@ sub _check_tag_url_privacy_breach {
 # css with @import
 sub detect_generic_privacy_breach {
     my ($block, $privacybreachhash, $file) = @_;
-    my %matchedkeyword = ();
+    my %matchedkeyword;
 
     # now check generic tag
   TYPE:
@@ -1964,14 +1960,14 @@ sub detect_generic_privacy_breach {
 
 sub detect_privacy_breach {
     my ($info, $file) = @_;
-    my %privacybreachhash = ();
+    my %privacybreachhash;
 
     # detect only in regular file
     unless($file->is_regular_file) {
         return;
     }
 
-    open(my $fd, '<:raw', $info->unpacked($file));
+    my $fd = $file->open(':raw');
 
     my $sfd = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); });
 
