@@ -42,7 +42,6 @@ use List::MoreUtils qw(any);
 
 use Lintian::Data;
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(is_ancestor_of);
 
 # This is a list of all tags that should be in every menu item.
 my @req_tags = qw(needs section title command);
@@ -70,7 +69,7 @@ my @needs_tag_vals = qw(x11 text vc);
 
 sub _menu_sections {
     my ($key, $val, $cur) = @_;
-    my $ret = undef;
+    my $ret;
     $ret = $cur = {} unless defined $cur;
     # $val is empty if this is just a root section
     $cur->{$val} = 1 if $val;
@@ -157,10 +156,10 @@ sub run {
     my ($pkg, $type, $info, $proc, $group) = @_;
     my $mdir = $info->lab_data_path('menu');
     my @menufiles;
-    for my $dir ("$mdir/lib", "$mdir/share") {
-        opendir(my $dirfd, $dir);
-        push(@menufiles, map { "$dir/$_" } readdir($dirfd));
-        closedir($dirfd);
+    for my $dirname (qw(usr/share/menu/ usr/lib/menu/)) {
+        if (my $dir = $info->index_resolved_path($dirname)) {
+            push(@menufiles, $dir->children);
+        }
     }
 
     # Find the desktop files in the package for verification.
@@ -184,23 +183,20 @@ sub run {
 
     # Verify all the desktop files.
     for my $desktop_file (@desktop_files) {
-        verify_desktop_file($desktop_file, $desktop_file, $pkg, $info);
+        verify_desktop_file($desktop_file, $pkg, $info);
     }
 
     # Now all the menu files.
     foreach my $menufile (@menufiles) {
-        next if -x $menufile; # don't try to parse executables
+        # Do not try to parse executables
+        next if $menufile->is_executable or not $menufile->is_open_ok;
 
-        my $basename = basename $menufile;
-        my $fullname = "usr/share/menu/$basename";
-        $fullname = "usr/lib/menu/$basename"
-          if $menufile =~ m,^\Q$mdir\E/lib/,;
-
-        next if $basename eq 'README'; # README is a special case
-        next if !is_ancestor_of($mdir, $menufile);
+        my $fullname = $menufile->name;
 
+        # README is a special case
+        next if $menufile->basename eq 'README';
         my $menufile_line ='';
-        open(my $fd, '<', $menufile);
+        my $fd = $menufile->open;
         # line below is commented out in favour of the while loop
         # do { $_=<IN>; } while defined && (m/^\s* \#/ || m/^\s*$/);
         while (<$fd>) {
@@ -505,30 +501,23 @@ sub verify_icon {
     }
 
     # Try the explicit location, and if that fails, try the standard path.
-    my $pkgroot = $info->unpacked;
-    my $iconfile = $info->unpacked($icon);
-    if (!-f $iconfile) {
-        $iconfile = $info->unpacked("usr/share/pixmaps/$icon");
-        if (!-f $iconfile) {
+    my $iconfile = $info->index_resolved_path($icon);
+    if (not $iconfile) {
+        $iconfile = $info->index_resolved_path("usr/share/pixmaps/$icon");
+        if (not $iconfile) {
             my $ginfo = $group->info;
             foreach my $depproc (@{ $ginfo->direct_dependencies($proc) }) {
                 my $dinfo = $depproc->info;
-                $pkgroot = $dinfo->unpacked;
-                $iconfile = $dinfo->unpacked($icon);
-                last if -f $iconfile;
-                $iconfile = $info->unpacked("usr/share/pixmaps/$icon");
-                last if -f $iconfile;
+                $iconfile = $dinfo->index_resolved_path($icon);
+                last if $iconfile;
+                $iconfile
+                  = $dinfo->index_resolved_path("usr/share/pixmaps/$icon");
+                last if $iconfile;
             }
         }
     }
 
-    # Last stat is a -f from above, reuse it
-    if (-e _) {
-        if (!is_ancestor_of($pkgroot, $iconfile)) {
-            # unsafe symlink
-            return;
-        }
-    } else {
+    if (not $iconfile or not $iconfile->is_open_ok) {
         tag 'menu-icon-missing', $icon;
         return;
     }
@@ -536,7 +525,7 @@ sub verify_icon {
     my $parse = 'XPM header';
     my $line;
 
-    open(my $fd, '<', $iconfile);
+    my $fd = $iconfile->open;
 
     do { defined($line = <$fd>) or goto parse_error; }
       until ($line =~ /\/\*\s*XPM\s*\*\//);
@@ -564,12 +553,10 @@ sub verify_icon {
 
 # Syntax-checks a .desktop file.
 sub verify_desktop_file {
-    my ($desktopfile, $file, $pkg, $info) = @_;
-    my ($line, $saw_first, $warned_cr);
-    my %vals;
-    my @pending;
-    open(my $fd, '<', $info->unpacked($file));
-    while (defined($line = <$fd>)) {
+    my ($file, $pkg, $info) = @_;
+    my ($saw_first, $warned_cr, %vals, @pending);
+    my $fd = $file->open;
+    while (my $line = <$fd>) {
         chomp $line;
         next if ($line =~ m/^\s*\#/ or $line =~ m/^\s*$/);
         if ($line =~ s/\r//) {
@@ -705,7 +692,7 @@ sub verify_desktop_file {
     # tags are present.
     if ($file =~ m,^usr/share/applications/, and defined $vals{'MimeType'}) {
         unless(defined $vals{'Exec'}
-            and $vals{'Exec'} =~ m,(?:^|[^%])%[fFuU]/,){
+            and $vals{'Exec'} =~ m,(?:^|[^%])%[fFuU],){
             tag 'desktop-mime-but-no-exec-code', $file;
         }
     }
