@@ -24,8 +24,6 @@ use strict;
 use warnings;
 use autodie;
 
-use File::Find qw();
-
 use List::MoreUtils qw(any);
 use Text::Levenshtein qw(distance);
 
@@ -56,32 +54,27 @@ my %dep5_renamed_fields        = (
 
 sub run {
     my (undef, undef, $info) = @_;
-    my $copyright_filename = $info->debfiles('copyright');
+    my $debian_dir = $info->index_resolved_path('debian/');
+    return if not $debian_dir;
+    my $copyright_path = $debian_dir->child('copyright');
 
-    if (-l $copyright_filename) {
-        tag 'debian-copyright-is-symlink';
-        return;
-    }
-
-    if (not -f $copyright_filename) {
+    if (not $copyright_path) {
         my @pkgs = $info->binaries;
         tag 'no-debian-copyright';
-        $copyright_filename = undef;
         if (scalar @pkgs == 1) {
-
             # If debian/copyright doesn't exist, and the only a single
             # binary package is built, there's a good chance that the
             # copyright file is available as
             # debian/<pkgname>.copyright.
-            $copyright_filename = $info->debfiles($pkgs[0] . '.copyright');
-            if (not -f $copyright_filename or -l $copyright_filename) {
-                $copyright_filename = undef;
-            }
+            $copyright_path = $debian_dir->child($pkgs[0] . '.copyright');
         }
+        return if not $copyright_path;
+    } elsif ($copyright_path->is_symlink) {
+        tag 'debian-copyright-is-symlink';
     }
 
-    if (defined($copyright_filename)) {
-        _check_dep5_copyright($info, $copyright_filename);
+    if ($copyright_path->is_open_ok) {
+        _check_dep5_copyright($info, $copyright_path);
     }
     return;
 }
@@ -150,8 +143,8 @@ sub _find_dep5_version {
 }
 
 sub _check_dep5_copyright {
-    my ($info, $copyright_filename) = @_;
-    my $contents = slurp_entire_file($copyright_filename);
+    my ($info, $copyright_path) = @_;
+    my $contents = $copyright_path->file_contents;
     my (@dep5, @lines);
 
     if (
@@ -292,7 +285,7 @@ sub _parse_dep5 {
             }else {
                 for (@short_licenses) {
                     $standalone_licenses{$_}             = $i;
-                    $short_licenses_seen{$short_license} = $i;
+                    $short_licenses_seen{$_} = $i;
                 }
             }
         }elsif (defined $files) {
@@ -311,6 +304,10 @@ sub _parse_dep5 {
             if (not @commas_in_files or $commas_in_files) {
                 my @wildcards = split /[\n\t ]+/, $files;
                 for my $wildcard (@wildcards) {
+                    $wildcard =~ s/^\s+|\s+$//g;
+                    if ($wildcard eq '') {
+                        next;
+                    }
                     my ($regex, $wildcard_error)= wildcard_to_regex($wildcard);
                     if (defined $wildcard_error) {
                         tag 'invalid-escape-sequence-in-dep5-copyright',
@@ -343,7 +340,7 @@ sub _parse_dep5 {
             }
             if ($found_license) {
                 for (@short_licenses) {
-                    $short_licenses_seen{$short_license} = $i;
+                    $short_licenses_seen{$_} = $i;
                     if (not defined($full_license)) {
                         $required_standalone_licenses{$_} = $i;
                     }
@@ -403,8 +400,16 @@ sub _parse_dep5 {
   LICENSE:
     while ((my $license, $i) = each %short_licenses_seen) {
         if ($license =~ m,\s,) {
-            tag 'space-in-std-shortname-in-dep5-copyright', $license,
-              "(paragraph at line $lines[$i]{'START-OF-PARAGRAPH'})";
+            if($license =~ m,[^ ]+ \s+ with \s+ (.*),x) {
+                my $exceptiontext = $1;
+                unless ($exceptiontext =~ m,[^ ]+ \s+ exception,x) {
+                    tag 'bad-exception-format-in-dep5-copyright', $license,
+                      "(paragraph at line $lines[$i]{'START-OF-PARAGRAPH'})";
+                }
+            }else {
+                tag 'space-in-std-shortname-in-dep5-copyright', $license,
+                  "(paragraph at line $lines[$i]{'START-OF-PARAGRAPH'})";
+            }
         }
         foreach my $bad_short_license ($BAD_SHORT_LICENSES->all) {
             my $value = $BAD_SHORT_LICENSES->value($bad_short_license);
@@ -422,8 +427,7 @@ sub _parse_dep5 {
 # parse a license block
 sub parse_license {
     my ($license_block, $line) = @_;
-    my $full_license  = undef;
-    my $short_license = undef;
+    my ($full_license, $short_license);
     return 0 unless defined($license_block);
     if ($license_block =~ m/\n/) {
         ($short_license, $full_license) = split /\n/, $license_block, 2;
@@ -497,20 +501,7 @@ sub wildcard_to_regex {
 
 sub get_all_files {
     my ($info) = @_;
-    # files with a trailing slash are directories
-    my @all_files = grep { not m,/$, } $info->sorted_index;
-    my $debfiles_root = $info->debfiles;
-    File::Find::find({
-            wanted => sub {
-                return unless -f $_;
-                my $dir = $File::Find::dir;
-                $dir =~ s,^\Q$debfiles_root\E(?:(?=/)|$),debian,;
-                push @all_files, "$dir/$_";
-            },
-        },
-        $debfiles_root
-    );
-    return @all_files;
+    return grep { $_->is_file } $info->sorted_index;
 }
 
 1;
