@@ -29,6 +29,7 @@ use Carp qw(croak);
 use Scalar::Util qw(blessed);
 
 use Lintian::Path;
+use Lintian::Path::FSInfo;
 use Lintian::Util qw(fail open_gz perm2oct normalize_pkg_path dequote_name);
 
 my %ROOT_INDEX_TEMPLATE = (
@@ -69,13 +70,13 @@ Lintian::Collect::Package - Lintian base interface to binary and source package
     my ($name, $type, $dir) = ('foobar', 'source', '/path/to/lab-entry');
     my $info = Lintian::Collect->new ($name, $type, $dir);
     my $filename = "etc/conf.d/$name.conf";
-    my $file = $info->index($filename);
-    if ($file && $file->is_regular_file) {
-        open(my $fd, '<', $info->unpacked($file));
+    my $file = $info->index_resolved_path($filename);
+    if ($file and $file->is_open_ok) {
+        my $fd = $info->open;
         # Use $fd ...
         close($fd);
     } elsif ($file) {
-        print "$file is available, but not a regular file\n";
+        print "$file is available, but is not a file or unsafe to open\n";
     } else {
         print "$file is missing\n";
     }
@@ -162,6 +163,15 @@ sub unpacked {
     #    (and duplicate said check in ::Binary::control and
     #    ::Source::debfiles).
     my $self = shift(@_);
+    my $f = $_[0] // '';
+
+    warnings::warnif(
+        'deprecated',
+        '[deprecated] The unpacked method is deprecated.  '
+          . "Consider using \$info->index_resolved_path('$f') instead."
+          . '  Called' # warnif appends " at <...>"
+    );
+
     return $self->_fetch_extracted_dir('unpacked', 'unpacked', @_);
 }
 
@@ -282,7 +292,19 @@ Needs-Info requirements for using I<index>: unpacked
 
 sub index {
     my ($self, $file) = @_;
-    return $self->_fetch_index_data('index', 'index', 'index-owner-id', $file);
+    if (my $cache = $self->{'index'}) {
+        return $cache->{$file}
+          if exists($cache->{$file});
+        return;
+    }
+    my $load_info = {
+        'field' => 'index',
+        'index_file' => 'index',
+        'index_owner_file' => 'index-owner-id',
+        'fs_root_sub' => 'unpacked',
+        'has_anchored_root_dir' => 1,
+    };
+    return $self->_fetch_index_data($load_info, $file);
 }
 
 =item sorted_index
@@ -308,6 +330,26 @@ sub sorted_index {
     return @{ $self->{sorted_index} };
 }
 
+=item index_resolved_path(PATH)
+
+Resolve PATH (relative to the root of the package) and return the
+L<entry|Lintian::Path> denoting the resolved path.
+
+The resolution is done using
+L<resolve_path|Lintian::Path/resolve_path([PATH])>.
+
+NB: For source packages, please see the
+L<"index"-caveat|Lintian::Collect::Source/index (FILE)>.
+
+Needs-Info requirements for using I<index_resolved_path>: L<Same as index|/index (FILE)>
+
+=cut
+
+sub index_resolved_path {
+    my ($self, $path) = @_;
+    return $self->index('')->resolve_path($path);
+}
+
 # Backing method for unpacked, debfiles and others; this is not a part of the
 # API.
 # sub _fetch_extracted_dir Needs-Info none
@@ -365,19 +407,20 @@ sub _fetch_extracted_dir {
 # Backing method for index and others; this is not a part of the API.
 # sub _fetch_index_data Needs-Info none
 sub _fetch_index_data {
-    my ($self, $field, $index, $indexown, $file) = @_;
-    if (my $cache = $self->{$field}) {
-        return $cache->{$file}
-          if exists $cache->{$file};
-        return;
-    }
+    my ($self, $load_info, $file) = @_;
+
+    my (%idxh, %children, $num_idx, %rhlinks, @sorted);
     my $base_dir = $self->base_dir;
-    my (%idxh, %children);
-    my $num_idx;
-    my %rhlinks;
-    my @sorted;
     local $_;
+    my $field = $load_info->{'field'};
+    my $index = $load_info->{'index_file'};
+    my $indexown = $load_info->{'index_owner_file'};
     my $idx = open_gz("$base_dir/${index}.gz");
+    my $fs_info = Lintian::Path::FSInfo->new(
+        '_collect' => $self,
+        '_collect_path_sub' => $load_info->{'fs_root_sub'},
+        'has_anchored_root_dir' => $load_info->{'as_anchored_root_dir'},
+    );
 
     if ($indexown) {
         $num_idx = open_gz("$base_dir/${indexown}.gz");
@@ -430,6 +473,7 @@ sub _fetch_index_data {
         # reason for storing it now is that we may need it during the
         # "hard-link fixup"-phase.
         $file{'name'} = $name = dequote_name($name);
+        $file{'_fs_info'} = $fs_info;
 
         $idxh{$name} = \%file;
 
@@ -458,6 +502,7 @@ sub _fetch_index_data {
             $cpy{'uid'} = 0;
             $cpy{'gid'} = 0;
         }
+        $cpy{'_fs_info'} = $fs_info;
         $idxh{''} = \%cpy;
     }
     if (%rhlinks) {
@@ -466,10 +511,8 @@ sub _fetch_index_data {
             # is still there.
             next unless exists $rhlinks{$file};
             my $e = $idxh{$file};
-            my %candidates = ();
             my @check = ($e->{name});
-            my @sorted;
-            my $target;
+            my (%candidates, @sorted, $target);
             while (my $current = pop @check) {
                 $candidates{$current} = 1;
                 foreach my $rdep (@{$rhlinks{$current}}) {
@@ -509,15 +552,19 @@ sub _fetch_index_data {
         # Add them in reverse order - entries in a dir are made
         # objects before the dir itself.
         if ($idxh{$file}->{type} eq 'd') {
-            my @children;
+            my %child_table;
             for my $cname (sort(@{ $children{$file} })) {
                 my $child = $idxh{$cname};
+                my $basename = $child->basename;
                 # Insert dirname here to share the same storage with
                 # the hash key
                 $child->{'dirname'} = $file;
-                push(@children, $child);
+                if (substr($basename, -1, 1) eq '/') {
+                    $basename = substr($basename, 0, -1);
+                }
+                $child_table{$basename} = $child;
             }
-            $idxh{$file}{children} = \@children;
+            $idxh{$file}{children} = \%child_table;
         }
         # Insert name here to share the same storage with the hash key
         $idxh{$file}{'name'} = $file;
