@@ -24,7 +24,6 @@ use warnings;
 use parent 'Lintian::Collect::Package';
 
 use Carp qw(croak);
-use Cwd();
 use Scalar::Util qw(blessed);
 
 use Lintian::Relation;
@@ -91,23 +90,21 @@ modules are also available.
 =item changelog
 
 Returns the changelog of the source package as a Parse::DebianChangelog
-object, or C<undef> if the changelog is a symlink or doesn't exist.  The
-debfiles collection script must have been run to create the changelog
-file, which this method expects to find in F<debfiles/changelog>.
+object, or C<undef> if the changelog cannot be resolved safely.
 
-Needs-Info requirements for using I<changelog>: L<Same as debfiles|/debfiles ([FILE])>
+Needs-Info requirements for using I<changelog>: L<Same as index_resolved_path|Lintian::Collect::Package/index_resolved_path(PATH)>
 
 =cut
 
 sub changelog {
     my ($self) = @_;
     return $self->{changelog} if exists $self->{changelog};
-    my $dch = $self->debfiles('changelog');
-    if (-l $dch || !-f $dch) {
-        $self->{changelog} = undef;
-    } else {
-        my %opts = (infile => $dch, quiet => 1);
+    my $dch = $self->index_resolved_path('debian/changelog');
+    if ($dch and $dch->is_open_ok) {
+        my %opts = (infile => $dch->fs_path, quiet => 1);
         $self->{changelog} = Parse::DebianChangelog->init(\%opts);
+    } else {
+        $self->{changelog} = undef;
     }
     return $self->{changelog};
 }
@@ -242,13 +239,10 @@ If FIELD is not given, return a hashref mapping field names to their
 values (in this case DEFAULT is ignored).  This hashref should not be
 modified.
 
-The debfiles collection script must have been run to make the
-F<debfiles/control> file available.
-
 NB: If a field from the "dsc" file itself is desired, please use
 L<field|Lintian::Collect/field> instead.
 
-Needs-Info requirements for using I<source_field>: L<Same as debfiles|/debfiles ([FILE])>
+Needs-Info requirements for using I<source_field>: L<Same as index_resolved_path|Lintian::Collect::Package/index_resolved_path(PATH)>
 
 =cut
 
@@ -280,8 +274,21 @@ Needs-Info requirements for using I<orig_index>: src-orig-index
 
 sub orig_index {
     my ($self, $file) = @_;
-    return $self->_fetch_index_data('orig-index', 'src-orig-index', undef,
-        $file);
+    if (my $cache = $self->{'orig_index'}) {
+        return $cache->{$file}
+          if exists($cache->{$file});
+        return;
+    }
+    my $load_info = {
+        'field' => 'orig_index',
+        'index_file' => 'src-orig-index',
+        'index_owner_file' => undef,
+        'fs_root_sub' => undef,
+        # source packages do not have anchored roots as they can be
+        # unpacked anywhere...
+        'has_anchored_root_dir' => 1,
+    };
+    return $self->_fetch_index_data($load_info, $file);
 }
 
 =item sorted_orig_index
@@ -305,8 +312,29 @@ sub sorted_orig_index {
     my ($self) = @_;
     # orig_index does all our work for us, so call it if
     # sorted_orig_index has not been created yet.
-    $self->orig_index('') unless exists $self->{'sorted_orig-index'};
-    return @{ $self->{'sorted_orig-index'} };
+    $self->orig_index('') unless exists($self->{'sorted_orig_index'});
+    return @{ $self->{'sorted_orig_index'} };
+}
+
+=item orig_index_resolved_path(PATH)
+
+Resolve PATH (relative to the root of the package) and return the
+L<entry|Lintian::Path> denoting the resolved path.
+
+The resolution is done using
+L<resolve_path|Lintian::Path/resolve_path([PATH])>.
+
+NB: If orig_index_resolved_path includes a debian packaging, it is was
+contained in upstream part of the source package (or the package is
+native).
+
+Needs-Info requirements for using I<orig_index_resolved_path>: L<Same as orig_index|/orig_index (FILE)>
+
+=cut
+
+sub orig_index_resolved_path {
+    my ($self, $path) = @_;
+    return $self->orig_index('')->resolve_path($path);
 }
 
 =item binary_field (PACKAGE[, FIELD[, DEFAULT]])
@@ -324,10 +352,7 @@ modified.
 If PACKAGE is not a binary built from this source, this returns
 C<undef> regardless of FIELD and DEFAULT.
 
-The debfiles collection script must have been run to make the
-F<debfiles/control> file available.
-
-Needs-Info requirements for using I<binary_field>: L<Same as debfiles|/debfiles ([FILE])>
+Needs-Info requirements for using I<binary_field>: L<Same as index_resolved_path|Lintian::Collect::Package/index_resolved_path(PATH)>
 
 =cut
 
@@ -348,28 +373,14 @@ sub binary_field {
 }
 
 # Internal method to load binary and source fields from
-# debfiles/control
-# sub _load_dctrl Needs-Info debfiles
+# debian/control
+# sub _load_dctrl Needs-Info unpacked
 sub _load_dctrl {
     my ($self) = @_;
     # Load the fields from d/control
-    my $dctrl = $self->debfiles('control');
+    my $dctrl = $self->index_resolved_path('debian/control');
     my $ok = 0;
-    if (-l $dctrl) {
-        # hmmm - this smells of trouble...
-        if (-e $dctrl) {
-            # it exists, but what does it point to?
-            my $droot = Cwd::abs_path($self->debfiles);
-            my $target = Cwd::abs_path($dctrl);
-            if ($droot && $target && $target =~ m,^$droot/,) {
-                # does not escape $droot, so it could work.
-                $ok = 1;
-            }
-        }
-    } else {
-        $ok = 1 if -e $dctrl;
-    }
-
+    $ok = 1 if $dctrl and $dctrl->is_open_ok;
     $self->{binary_names} = [];
     unless ($ok) {
         # Bad file, assume the package and field does not exist.
@@ -377,10 +388,9 @@ sub _load_dctrl {
         $self->{source_field} = {};
         return;
     }
-    my @control_data;
-    my %packages;
-
-    eval {@control_data = read_dpkg_control($dctrl);};
+    my (@control_data, %packages);
+    my $fs_path = $dctrl->fs_path;
+    eval {@control_data = read_dpkg_control($fs_path);};
     if ($@) {
         # If it is a syntax error, ignore it (we emit
         # syntax-error-in-control-file in this case via
@@ -545,6 +555,10 @@ sub relation_noarch {
 
 =item debfiles ([FILE])
 
+B<This method is deprecated>.  Consider using
+L<index_resolved_path(PATH)|Lintian::Collect::Package/index_resolved_path(PATH)>
+instead, which returns L<Lintian::Path> objects.
+
 Returns the path to FILE in the debian dir of the extracted source
 package.  FILE must be relative to the root of the debian dir and
 should be without leading slash (and and without "./").  If FILE is
@@ -566,9 +580,17 @@ sub debfiles {
     ## no critic (Subroutines::RequireArgUnpacking)
     # - see L::Collect::unpacked for why
     my $self = shift(@_);
+    my $f = $_[0] // '';
     if (defined($_[0]) && blessed($_[0])) {
         croak('debfiles does not accept blessed objects');
     }
+    warnings::warnif(
+        'deprecated',
+        '[deprecated] The debfiles method is deprecated.  '
+          . "Consider using \$info->index_resolved_path(\"debian/$f\") instead."
+          . '  Called' # warnif appends " at <...>"
+    );
+
     return $self->_fetch_extracted_dir('debfiles', 'debfiles', @_);
 }
 
@@ -618,7 +640,21 @@ Needs-Info requirements for using I<index>: unpacked
 
 sub index {
     my ($self, $file) = @_;
-    return $self->_fetch_index_data('index', 'index', undef, $file);
+    if (my $cache = $self->{'index'}) {
+        return $cache->{$file}
+          if exists($cache->{$file});
+        return;
+    }
+    my $load_info = {
+        'field' => 'index',
+        'index_file' => 'index',
+        'index_owner_file' => undef,
+        'fs_root_sub' => 'unpacked',
+        # source packages do not have anchored roots as they can be
+        # unpacked anywhere...
+        'has_anchored_root_dir' => 0,
+    };
+    return $self->_fetch_index_data($load_info, $file);
 }
 
 =item is_non_free
