@@ -26,6 +26,7 @@ use autodie;
 
 use Carp qw(croak);
 use Cwd qw(abs_path);
+use Errno qw(ENOENT);
 use Exporter qw(import);
 
 use constant {
@@ -79,17 +80,22 @@ BEGIN {
           drain_pipe
           signal_number2name
           dequote_name
+          load_state_cache
+          find_backlog
+          unix_locale_split
           $PKGNAME_REGEX),
         @{ $EXPORT_TAGS{constants} });
 }
 
-use Encode ();
-use FileHandle;
-use Lintian::Command qw(spawn);
 use Digest::MD5;
 use Digest::SHA;
+use Encode ();
+use FileHandle;
 use Scalar::Util qw(openhandle);
 
+use Lintian::Command qw(spawn);
+use Lintian::Relation::Version qw(versions_equal versions_comparator);
+
 =head1 NAME
 
 Lintian::Util - Lintian utility functions
@@ -307,7 +313,7 @@ Where %d is the line number of the issue and %s is one of:
 
 The field appeared twice in the paragraph.
 
-=item Continuation line outside a paragraph
+=item Continuation line outside a paragraph (maybe line %d should be " .")
 
 A continuation line appears outside a paragraph - usually caused by an
 unintended empty line before it.
@@ -386,13 +392,13 @@ sub visit_dpkg_paragraph {
     while (<$CONTROL>) {
         chomp;
 
-        if (/^\#/) {
+        if (substr($_, 0, 1) eq '#') {
             next unless $flags & DCTRL_NO_COMMENTS;
             die "syntax error at line $.: Comments are not allowed.\n";
         }
 
         # empty line?
-        if ((!$debconf && m/^\s*$/) or ($debconf && $_ eq '')) {
+        if ($_ eq '' || (!$debconf && m/^\s*$/)) {
             if ($open_section) { # end of current section
                 # pass the current section to the handler
                 $code->($section, $lines);
@@ -543,7 +549,8 @@ sub visit_dpkg_paragraph {
             $open_section
               or die join(q{ },
                 "syntax error at line $.:",
-                "Continuation line outside a paragraph.\n");
+                'Continuation line outside a paragraph (maybe line',
+                ($. - 1), qq{should be " .").\n});
 
             # Policy: Many fields' values may span several lines; in this case
             # each continuation line must start with a space or a tab.  Any
@@ -1030,7 +1037,7 @@ If the tool cannot be found, this sub will cause a trappable error.
 =cut
 
 {
-    my %_CACHE = ();
+    my %_CACHE;
 
     sub locate_helper_tool {
         my ($toolname) = @_;
@@ -1099,10 +1106,12 @@ sub strip (_) { ## no critic (Subroutines::RequireFinalReturn)
     if (defined wantarray) {
         # perl 5.14 s///r would have been useful here.
         my ($arg) = @_;
-        $arg =~ s/^\s++|\s++$//g;
-        return $arg;
+        $arg =~ s/^\s++//;
+        # unpack 'A*' is faster than s/\s++$//
+        return unpack('A*', $arg);
     }
-    $_[0] =~ s/^\s++|\s++$//g;
+    $_[0] =~ s/^\s++//;
+    $_[0] = unpack('A*', $_[0]);
     # void context, so no return needed here.
 }
 
@@ -1121,12 +1130,10 @@ sub lstrip (_) { ## no critic (Subroutines::RequireFinalReturn)
 # prototype for default to $_
 sub rstrip (_) {  ## no critic (Subroutines::RequireFinalReturn)
     if (defined wantarray) {
-        # perl 5.14 s///r would have been useful here.
-        my ($arg) = @_;
-        $arg =~ s/\s++$//g;
-        return $arg;
+        # unpack 'A*' is faster than s/\s++$//
+        return unpack('A*', $_[0]);
     }
-    $_[0] =~ s/\s++$//;
+    $_[0] = unpack('A*', $_[0]);
     # void context, so no return needed here.
 }
 
@@ -1161,8 +1168,12 @@ sub dequote_name {
     my ($name, $slsd) = @_;
     $slsd = 1 unless defined $slsd;
     $name =~ s,^\.?/,, if $slsd;
-    $name =~ s/(\G|[^\\](?:\\\\)*)\\(\d{3})/"$1" . chr(oct $2)/ge;
-    $name =~ s/\\\\/\\/g;
+    # Optimise for the case where the filename does not contain
+    # backslashes.  It is a fairly rare to see that in practise.
+    if (index($name, '\\') > -1) {
+        $name =~ s/(\G|[^\\](?:\\\\)*)\\(\d{3})/"$1" . chr(oct $2)/ge;
+        $name =~ s/\\\\/\\/g;
+    }
     return $name;
 }
 
@@ -1360,6 +1371,111 @@ sub is_ancestor_of {
     return 0;
 }
 
+=item unix_locale_split(STR)
+
+Read STR as a locale code (e.g. en_GB.UTF-8) and return a list of
+locale codes ordered by preference.  As an example, en_GB.UTF-8
+might return
+
+  en_GB
+  en
+
+Note encoding I<is ignored> as all Lintian files are always encoded in
+UTF-8.
+
+B<Special cases>: The "C" or "POSIX" locale returns the empty list.
+Other strings that do not match the expected format causes a trappable
+error.
+
+=cut
+
+sub unix_locale_split {
+    my ($str) = @_;
+    my ($locale_part, undef) = split(m/\./, $str, 2);
+    my @parts;
+    if ($locale_part eq 'POSIX' or $locale_part eq 'C') {
+        return;
+    }
+
+    if ($locale_part !~ m/\A [a-z0-9-]+ (?: _ [a-z0-9-]+)* \Z/xsmi) {
+        croak("Cannot parse $str as a locale string");
+    }
+
+    while ($locale_part ne '') {
+        my $end;
+        push(@parts, $locale_part);
+        $end = rindex($locale_part, '_');
+        last if ($end == -1);
+        $locale_part = substr($locale_part, 0, $end);
+    }
+
+    return @parts;
+}
+
+=item load_state_cache(STATE_DIR)
+
+[Reporting tools only] Load the state cache from STATE_DIR.
+
+=cut
+
+sub load_state_cache {
+    my ($state_dir) = @_;
+    my $state_file = "$state_dir/state-cache";
+    my $state = {};
+    my $fd;
+    require YAML::Any;
+    eval {open($fd, '<', $state_file);};
+    if (my $err = $@) {
+        if ($err->errno != ENOENT) {
+            # Present, but unreadable for some reason
+            die($err);
+        }
+        # Not present; presume empty
+        return $state;
+    }
+    eval {$state = YAML::Any::Load(slurp_entire_file($fd, 1));};
+    # Not sure what Load does in case of issues; perldoc YAML says
+    # very little about it.  Based on YAML::Error, I guess it will
+    # write stuff to STDERR and use die/croak, but it remains a
+    # guess.
+    if (my $err = $@) {
+        die("$state_file was invalid; please fix or remove it.\n$err");
+    }
+    $state //= {};
+
+    if (ref($state) ne 'HASH') {
+        die("$state_file was invalid; please fix or remove it.");
+    }
+    close($fd);
+    return $state;
+}
+
+=item find_backlog(LINTIAN_VERSION, STATE)
+
+[Reporting tools only] Given the current lintian version and the
+harness state, return a list of group ids that are part of the
+backlog.  The list is sorted based on what version of Lintian
+processed the package.
+
+=cut
+
+sub find_backlog {
+    my ($lintian_version, $state) = @_;
+    my (@list, @sorted);
+    for my $group_id (keys(%{$state})) {
+        my $last_version = '0';
+        if (exists($state->{$group_id}{'last-processed-by'})) {
+            $last_version = $state->{$group_id}{'last-processed-by'};
+        }
+        push(@list, [$group_id, $last_version])
+          if not versions_equal($last_version, $lintian_version);
+    }
+    @sorted = map { $_->[0] }
+      sort { versions_comparator($a->[1], $b->[1]) || $a->[0] cmp $b->[0] }
+      @list;
+    return @sorted;
+}
+
 =back
 
 =head1 SEE ALSO
